@@ -1,11 +1,12 @@
 import os
 import shutil
 import subprocess
-from cookiecutter.main import cookiecutter
 from pathlib import Path
+from urllib.parse import urlparse
 
 import click
 import yaml
+from cookiecutter.main import cookiecutter
 
 from ctfcli.utils.challenge import (
     create_challenge,
@@ -20,19 +21,35 @@ from ctfcli.utils.config import (
     get_project_path,
     load_config,
 )
-from ctfcli.utils.spec import (
-    CHALLENGE_SPEC_DOCS,
-    blank_challenge_spec,
-)
+from ctfcli.utils.deploy import DEPLOY_HANDLERS
+from ctfcli.utils.spec import CHALLENGE_SPEC_DOCS, blank_challenge_spec
+from ctfcli.utils.templates import get_template_dir
 
 
 class Challenge(object):
-    def new(self, type):
-        path = Path(get_base_path())
-        if os.sep not in type:
-            type += os.sep + "default"
-        path = path / "templates" / type
-        cookiecutter(str(path))
+    def new(self, type="blank"):
+        if type == "blank":
+            path = Path(get_base_path())
+            path = path / "templates" / type / "default"
+            cookiecutter(str(path))
+        else:
+            # Check if we're referencing an installed template
+            template_dir = Path(get_template_dir())
+            template_path = template_dir / type
+
+            if template_path.is_dir():  # If we found a template directory, use it
+                cookiecutter(str(template_path))
+            else:  # If we didn't, use a built in template
+                path = Path(get_base_path())
+                if os.sep not in type:
+                    type += os.sep + "default"
+                path = path / "templates" / type
+                cookiecutter(str(path))
+
+    def templates(self):
+        from ctfcli.cli.templates import Templates
+
+        Templates().list()
 
     def add(self, repo):
         config = load_config()
@@ -136,15 +153,28 @@ class Challenge(object):
         sync_challenge(challenge=challenge)
         click.secho(f"Success!", fg="green")
 
-    def update(self):
+    def update(self, challenge=None):
         config = load_config()
         challenges = dict(config["challenges"])
         for folder, url in challenges.items():
+            if challenge and challenge != folder:
+                continue
             if url.endswith(".git"):
                 click.echo(f"Cloning {url} to {folder}")
                 subprocess.call(["git", "init"], cwd=folder)
                 subprocess.call(["git", "remote", "add", "origin", url], cwd=folder)
-                subprocess.call(["git", "pull"], cwd=folder)
+                subprocess.call(["git", "add", "-A"], cwd=folder)
+                subprocess.call(
+                    ["git", "commit", "-m", "Persist local changes (ctfcli)"],
+                    cwd=folder,
+                )
+                subprocess.call(
+                    ["git", "pull", "--allow-unrelated-histories", "origin", "master"],
+                    cwd=folder,
+                )
+                subprocess.call(["git", "mergetool"], cwd=folder)
+                subprocess.call(["git", "clean", "-f"], cwd=folder)
+                subprocess.call(["git", "commit", "--no-edit"], cwd=folder)
                 shutil.rmtree(str(Path(folder) / ".git"))
             else:
                 click.echo(f"Skipping {url} - {folder}")
@@ -205,3 +235,49 @@ class Challenge(object):
             path = path / "challenge.yml"
 
         lint_challenge(path)
+
+    def deploy(self, challenge, host=None):
+        if challenge is None:
+            challenge = os.getcwd()
+
+        path = Path(challenge)
+
+        if path.name.endswith(".yml") is False:
+            path = path / "challenge.yml"
+
+        challenge = load_challenge(path)
+        image = challenge.get("image")
+        target_host = host or challenge.get("host") or input("Target host URI: ")
+        if image is None:
+            click.secho(
+                "This challenge can't be deployed because it doesn't have an associated image",
+                fg="red",
+            )
+            return
+        if bool(target_host) is False:
+            click.secho(
+                "This challenge can't be deployed because there is no target host to deploy to",
+                fg="red",
+            )
+            return
+        url = urlparse(target_host)
+
+        if bool(url.netloc) is False:
+            click.secho(
+                "Provided host has no URI scheme. Provide a URI scheme like ssh:// or registry://",
+                fg="red",
+            )
+            return
+
+        status, domain, port = DEPLOY_HANDLERS[url.scheme](
+            challenge=challenge, host=target_host
+        )
+
+        if status:
+            click.secho(
+                f"Challenge deployed at {domain}:{port}", fg="green",
+            )
+        else:
+            click.secho(
+                f"An error occured during deployment", fg="red",
+            )
