@@ -1,8 +1,7 @@
 from pathlib import Path
 
-import yaml
-
 import click
+import yaml
 
 from .config import generate_session
 
@@ -159,7 +158,7 @@ def sync_challenge(challenge):
         r.raise_for_status()
 
     # Unhide challenge depending upon the value of "state" in spec
-    data = {"state": "hidden"}
+    data = {"state": "visible"}
     if challenge.get("state"):
         if challenge["state"] in ["hidden", "visible"]:
             data["state"] = challenge["state"]
